package com.ningdatech.pmapi.meeting.manage;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.common.util.WordUtil;
import com.ningdatech.pmapi.expert.entity.ExpertUserFullInfo;
import com.ningdatech.pmapi.expert.service.IExpertUserFullInfoService;
import com.ningdatech.pmapi.meeting.entity.domain.Meeting;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingExpert;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingInnerProject;
import com.ningdatech.pmapi.meeting.entity.domain.MeetingOuterProject;
import com.ningdatech.pmapi.meeting.entity.dto.ExpertFeeExportDTO;
import com.ningdatech.pmapi.meeting.entity.dto.ExpertReviewTableDTO;
import com.ningdatech.pmapi.meeting.service.IMeetingExpertService;
import com.ningdatech.pmapi.meeting.service.IMeetingInnerProjectService;
import com.ningdatech.pmapi.meeting.service.IMeetingOuterProjectService;
import com.ningdatech.pmapi.meeting.service.IMeetingService;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sms.utils.DateUtil;
import freemarker.template.TemplateException;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * <p>
 * ExpertExportManage
 * </p>
 *
 * @author WendyYang
 * @since 2023/7/24
 **/
@Slf4j
@Component
@AllArgsConstructor
public class ExpertExportManage {

    private final IMeetingService meetingService;
    private final IMeetingExpertService meetingExpertService;
    private final IMeetingInnerProjectService meetingInnerProjectService;
    private final IMeetingOuterProjectService meetingOuterProjectService;
    private final IExpertUserFullInfoService expertUserInfoService;
    private final IProjectService projectService;

    public void exportFeeForExpert(Long meetingId, HttpServletResponse response) {
        final String template = "专家费.ftl";
        String fileName = UUID.randomUUID().toString(true) + ".doc";
        try {
            Meeting meeting = meetingService.getById(meetingId);
            ExpertFeeExportDTO data = new ExpertFeeExportDTO();
            data.setMeetingTime(DateUtil.localDateTimeFormat(meeting.getStartTime(), "yyyy-MM-dd HH:mm"));
            data.setMeetingAddress(meeting.getMeetingAddress());
            List<String> projectNames = new ArrayList<>();
            if (meeting.getIsInnerProject()) {
                List<MeetingInnerProject> inners = meetingInnerProjectService.listByMeetingId(meetingId);
                List<Long> projectIds = CollUtils.fieldList(inners, MeetingInnerProject::getProjectId);
                List<Project> projects = projectService.listByIds(projectIds);
                projects.forEach(w -> projectNames.add(w.getProjectName()));
            } else {
                List<MeetingOuterProject> inners = meetingOuterProjectService.listByMeetingId(meetingId);
                inners.forEach(w -> projectNames.add(w.getProjectName()));
            }
            data.setProjectName(StrUtil.join("、", projectNames));
            data.setTableData(new ArrayList<>());
            // 设置专家信息
            List<MeetingExpert> experts = meetingExpertService.listAgreedExperts(meetingId);
            List<Long> expertIds = CollUtils.fieldList(experts, MeetingExpert::getExpertId);
            List<ExpertUserFullInfo> expertInfos = expertUserInfoService.listByUserId(expertIds);
            Map<Long, ExpertUserFullInfo> expertMap = CollUtils.listToMap(expertInfos, ExpertUserFullInfo::getUserId);
            AtomicInteger integer = new AtomicInteger(0);
            experts.forEach(w -> {
                ExpertFeeExportDTO.ExpertInfoDTO expert = new ExpertFeeExportDTO.ExpertInfoDTO();
                expert.setNo(integer.incrementAndGet());
                expert.setFee(BigDecimal.valueOf(500));
                expert.setName(w.getExpertName());
                expert.setMobile(w.getMobile());
                ExpertUserFullInfo expertUser = expertMap.get(w.getExpertId());
                if (expertUser != null) {
                    expert.setIdcard(expertUser.getIdCard());
                    expert.setBankNo(expertUser.getBankNo());
                    expert.setBank(expertUser.getBank());
                    expert.setCompany(expertUser.getCompany());
                    expert.setSex(expertUser.getGender());
                }
                data.getTableData().add(expert);
            });
            WordUtil.export(fileName, data, response, template);
        } catch (TemplateException | IOException e) {
            log.error("专家费用单导出异常：{}", meetingId, e);
            throw BizException.wrap("专家费用单导出失败");
        }
    }

    public void expertReviewTable(Long meetingId, HttpServletResponse response) {
        final String template = "专家评审单.ftl";
        String fileName = UUID.randomUUID().toString(true) + ".doc";
        try {
            Meeting meeting = meetingService.getById(meetingId);
            ExpertReviewTableDTO data = new ExpertReviewTableDTO();
            data.setMeetingTime(DateUtil.localDateTimeFormat(meeting.getStartTime(), "yyyy年M月d日"));
            data.setMeetingAddress(meeting.getMeetingAddress());
            data.setHoldOrg(meeting.getHoldOrg());
            List<String> projectNames = new ArrayList<>();
            if (meeting.getIsInnerProject()) {
                List<MeetingInnerProject> inners = meetingInnerProjectService.listByMeetingId(meetingId);
                List<Long> projectIds = CollUtils.fieldList(inners, MeetingInnerProject::getProjectId);
                List<Project> projects = projectService.listByIds(projectIds);
                projects.forEach(w -> projectNames.add(w.getProjectName()));
            } else {
                List<MeetingOuterProject> inners = meetingOuterProjectService.listByMeetingId(meetingId);
                inners.forEach(w -> projectNames.add(w.getProjectName()));
            }
            data.setProjectName(StrUtil.join("、", projectNames));
            WordUtil.export(fileName, data, response, template);
        } catch (TemplateException | IOException e) {
            log.error("专家评审单导出异常：{}", meetingId, e);
            throw BizException.wrap("专家评审单导出失败");
        }
    }

}
